package be.iminds.ilabt.jfed.git;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.Optional;

/**
 * GitFetcherCommand: contains all info about:
 *   - what git repo
 *   - what data to get from it
 *   - where to put it
 */
public class GitFetcherCommand {
    @Nonnull private final String gitUrl;
    @Nullable private final String repoSubDir;
    @Nullable private final String branch;

    @Nullable private final String username;
    @Nullable private final String password;

    @Nullable private final String privateKeyPem;

    @Nullable private final File targetDir;
    private final boolean allowCreateDir;
    private final boolean overwriteFilesInDir;

    public GitFetcherCommand(@Nonnull  String gitUrl,
                             @Nullable String repoSubDir,
                             @Nullable String branch,
                             @Nullable String username,
                             @Nullable String password,
                             @Nullable String privateKeyPem,
                             @Nullable File targetDir,
                             boolean allowCreateDir,
                             boolean overwriteFilesInDir) {
        this.gitUrl = gitUrl;
        this.repoSubDir = repoSubDir;
        this.branch = branch;
        this.username = username;
        this.password = password;
        this.privateKeyPem = privateKeyPem;
        this.targetDir = targetDir;
        this.allowCreateDir = allowCreateDir;
        this.overwriteFilesInDir = overwriteFilesInDir;
    }

    public static File createTempAnsibleDir() {
        try {
            return Files.createTempDirectory("git-repo").toFile();
        } catch (IOException e) {
            throw new RuntimeException("Could not create temporary dir for git repo files.", e);
        }
    }


    public static Optional<GitFetcherCommand> fromString(@Nonnull String command) {
        return fromString(command, null);
    }

    /**
     * @return The GitFetcherCommand in the string, unless the string does not include one.
     */
    public static Optional<GitFetcherCommand> fromString(@Nonnull String command,
                                                         @Nullable File targetDir) {
        if (targetDir == null)
            targetDir = createTempAnsibleDir();

        boolean isSpecialGit = command.trim().startsWith("git ");
        boolean isSpecialGithub = command.trim().startsWith("github ");

        if (!isSpecialGit && !isSpecialGithub)
            return Optional.empty();

        String[] gitSourceSpecSplit = command.trim().split(" ");
        if (gitSourceSpecSplit.length > 4)
            throw new IllegalArgumentException(
                    "'"+command+"' has too many parts. Syntax:" +
                            " git[hub] https://<githubTLD>/<org>/<repo>.git [<subdir> [<branch>]]");
        if (gitSourceSpecSplit.length < 2)
            throw new IllegalArgumentException(
                    "'"+command+"' has too few parts. Syntax:" +
                            " git[hub] https://<githubTLD>/<org>/<repo>.git [<subdir> [<branch>]]");

        assert gitSourceSpecSplit[0].trim().startsWith("git");
        String gitUrl = gitSourceSpecSplit[1];
        String subDir = gitSourceSpecSplit.length > 2 ? gitSourceSpecSplit[2] : null;
        String branch = gitSourceSpecSplit.length > 3 ? gitSourceSpecSplit[3] : null;

        if (subDir != null && !subDir.endsWith("/"))
            subDir += "/";

        return Optional.of(new GitFetcherCommandBuilder().setGitUrl(gitUrl).setRepoSubDir(subDir).setBranch(branch).setUsername(null).setPassword(null).setPrivateKeyPem(null).setTargetDir(targetDir).setAllowCreateDir(false).setOverwriteFilesInDir(false).build());
    }

    @Nonnull
    public String getGitUrl() {
        return gitUrl;
    }

    @Nullable
    public String getRepoSubDir() {
        return repoSubDir;
    }

    @Nullable
    public String getBranch() {
        return branch;
    }

    @Nullable
    public String getUsername() {
        return username;
    }

    @Nullable
    public String getPassword() {
        return password;
    }

    @Nullable
    public String getPrivateKeyPem() {
        return privateKeyPem;
    }

    @Nullable
    public File getTargetDir() {
        return targetDir;
    }

    public boolean isAllowCreateDir() {
        return allowCreateDir;
    }

    public boolean isOverwriteFilesInDir() {
        return overwriteFilesInDir;
    }
}
